@@ -244,7 +244,7 @@ void Sweep::FillAdvancingFront(SweepContext& tcx, Node& n)
 
   // Fill right basins
   if (n.next && n.next->next) {
-    float angle = BasinAngle(n);
+    double angle = BasinAngle(n);
     if (angle < PI_3div4) {
       FillBasin(tcx, n);
     }
@@ -274,18 +274,18 @@ bool Sweep::LargeHole_DontFill(Node* node) {
 }
 
 bool Sweep::AngleExceeds90Degrees(Point* origin, Point* pa, Point* pb) {
-  float angle = Angle(*origin, *pa, *pb);
+  double angle = Angle(*origin, *pa, *pb);
   bool exceeds90Degrees = ((angle > PI_div2) || (angle < -PI_div2));
   return exceeds90Degrees;
 }
 
 bool Sweep::AngleExceedsPlus90DegreesOrIsNegative(Point* origin, Point* pa, Point* pb) {
-  float angle = Angle(*origin, *pa, *pb);
+  double angle = Angle(*origin, *pa, *pb);
   bool exceedsPlus90DegreesOrIsNegative = (angle > PI_div2) || (angle < 0);
   return exceedsPlus90DegreesOrIsNegative;
 }
 
-float Sweep::Angle(Point& origin, Point& pa, Point& pb) {
+double Sweep::Angle(Point& origin, Point& pa, Point& pb) {
   /* Complex plane
    * ab = cosA +i*sinA
    * ab = (ax + ay*i)(bx + by*i) = (ax*bx + ay*by) + i(ax*by-ay*bx)
@@ -294,26 +294,26 @@ float Sweep::Angle(Point& origin, Point& pa, Point& pb) {
    * Where x = ax*bx + ay*by
    *       y = ax*by - ay*bx
    */
-  float px = origin.x;
-  float py = origin.y;
-  float ax = pa.x- px;
-  float ay = pa.y - py;
-  float bx = pb.x - px;
-  float by = pb.y - py;
-  float x = ax * by - ay * bx;
-  float y = ax * bx + ay * by;
-  float angle = atan2(x, y);
+  double px = origin.x;
+  double py = origin.y;
+  double ax = pa.x- px;
+  double ay = pa.y - py;
+  double bx = pb.x - px;
+  double by = pb.y - py;
+  double x = ax * by - ay * bx;
+  double y = ax * bx + ay * by;
+  double angle = atan2(x, y);
   return angle;
 }
 
-float Sweep::BasinAngle(Node& node)
+double Sweep::BasinAngle(Node& node)
 {
-  float ax = node.point->x - node.next->next->point->x;
-  float ay = node.point->y - node.next->next->point->y;
+  double ax = node.point->x - node.next->next->point->x;
+  double ay = node.point->y - node.next->next->point->y;
   return atan2(ay, ax);
 }
 
-float Sweep::HoleAngle(Node& node)
+double Sweep::HoleAngle(Node& node)
 {
   /* Complex plane
    * ab = cosA +i*sinA
@@ -323,10 +323,10 @@ float Sweep::HoleAngle(Node& node)
    * Where x = ax*bx + ay*by
    *       y = ax*by - ay*bx
    */
-  float ax = node.next->point->x - node.point->x;
-  float ay = node.next->point->y - node.point->y;
-  float bx = node.prev->point->x - node.point->x;
-  float by = node.prev->point->y - node.point->y;
+  double ax = node.next->point->x - node.point->x;
+  double ay = node.next->point->y - node.point->y;
+  double bx = node.prev->point->x - node.point->x;
+  double by = node.prev->point->y - node.point->y;
   return atan2(ax * by - ay * bx, ax * bx + ay * by);
 }
 
@@ -393,36 +393,36 @@ bool Sweep::Legalize(SweepContext& tcx, Triangle& t)
 
 bool Sweep::Incircle(Point& pa, Point& pb, Point& pc, Point& pd)
 {
-  float adx = pa.x - pd.x;
-  float ady = pa.y - pd.y;
-  float bdx = pb.x - pd.x;
-  float bdy = pb.y - pd.y;
+  double adx = pa.x - pd.x;
+  double ady = pa.y - pd.y;
+  double bdx = pb.x - pd.x;
+  double bdy = pb.y - pd.y;
 
-  float adxbdy = adx * bdy;
-  float bdxady = bdx * ady;
-  float oabd = adxbdy - bdxady;
+  double adxbdy = adx * bdy;
+  double bdxady = bdx * ady;
+  double oabd = adxbdy - bdxady;
 
   if (oabd <= 0)
     return false;
 
-  float cdx = pc.x - pd.x;
-  float cdy = pc.y - pd.y;
+  double cdx = pc.x - pd.x;
+  double cdy = pc.y - pd.y;
 
-  float cdxady = cdx * ady;
-  float adxcdy = adx * cdy;
-  float ocad = cdxady - adxcdy;
+  double cdxady = cdx * ady;
+  double adxcdy = adx * cdy;
+  double ocad = cdxady - adxcdy;
 
   if (ocad <= 0)
     return false;
 
-  float bdxcdy = bdx * cdy;
-  float cdxbdy = cdx * bdy;
+  double bdxcdy = bdx * cdy;
+  double cdxbdy = cdx * bdy;
 
-  float alift = adx * adx + ady * ady;
-  float blift = bdx * bdx + bdy * bdy;
-  float clift = cdx * cdx + cdy * cdy;
+  double alift = adx * adx + ady * ady;
+  double blift = bdx * bdx + bdy * bdy;
+  double clift = cdx * cdx + cdy * cdy;
 
-  float det = alift * (bdxcdy - cdxbdy) + blift * ocad + clift * oabd;
+  double det = alift * (bdxcdy - cdxbdy) + blift * ocad + clift * oabd;
 
   return det > 0;
 }
@@ -548,7 +548,7 @@ void Sweep::FillBasinReq(SweepContext& tcx, Node* node)
 
 bool Sweep::IsShallow(SweepContext& tcx, Node& node)
 {
-  float height;
+  double height;
 
   if (tcx.basin.left_highest) {
     height = tcx.basin.left_node->point->y - node.point->y;
