@@ -46,6 +46,8 @@
 #include <QtPositioning/QGeoShape>
 #include <QtPositioning/QGeoRectangle>
 #include <QtPositioning/QGeoCircle>
+#include <QtPositioning/QGeoPath>
+#include <QtQml/QJSValue>
 #include <QVariant>
 
 class LocationSingleton : public QObject
@@ -71,8 +73,12 @@ public:
     Q_INVOKABLE QGeoCircle circle() const;
     Q_INVOKABLE QGeoCircle circle(const QGeoCoordinate &center, qreal radius = -1.0) const;
 
+    Q_INVOKABLE QGeoPath path() const;
+    Q_INVOKABLE QGeoPath path(const QJSValue &value, qreal width = 0.0) const;
+
     Q_INVOKABLE QGeoCircle shapeToCircle(const QGeoShape &shape) const;
     Q_INVOKABLE QGeoRectangle shapeToRectangle(const QGeoShape &shape) const;
+    Q_INVOKABLE QGeoPath shapeToPath(const QGeoShape &shape) const;
 };
 
 #endif // LOCATIONSINGLETON_H
