@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtPositioning module of the Qt Toolkit.
 **
@@ -10,30 +10,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -47,7 +45,7 @@
 #include <QtQml/qqml.h>
 #include <qnmeapositioninfosource.h>
 #include <QFile>
-#include <QTcpSocket>
+#include <QtNetwork/QTcpSocket>
 #include <QTimer>
 
 QT_BEGIN_NAMESPACE
@@ -107,7 +105,7 @@ QT_BEGIN_NAMESPACE
     }
     \endcode
 
-    The \l{flickr}{Flickr} example application shows how to use
+    The \l{geoflickr}{GeoFlickr} example application shows how to use
     a PositionSource in your application to retrieve local data for users
     from a REST web service.
 
@@ -115,6 +113,21 @@ QT_BEGIN_NAMESPACE
 
 */
 
+/*!
+    \qmlsignal PositionSource::updateTimeout()
+
+    If \l update() was called, this signal is emitted if the current position could not be
+    retrieved within a certain amount of time.
+
+    If \l start() was called, this signal is emitted if the position engine determines that
+    it is not able to provide further regular updates.
+
+    \since Qt Positioning 5.5
+
+    \sa QGeoPositionInfoSource::updateTimeout()
+*/
+
+
 QDeclarativePositionSource::QDeclarativePositionSource()
 :   m_positionSource(0), m_preferredPositioningMethods(NoPositioningMethods), m_nmeaFile(0),
     m_nmeaSocket(0), m_active(false), m_singleUpdate(false), m_updateInterval(0),
@@ -175,6 +188,8 @@ void QDeclarativePositionSource::setName(const QString &newName)
                 this, SLOT(positionUpdateReceived(QGeoPositionInfo)));
         connect(m_positionSource, SIGNAL(error(QGeoPositionInfoSource::Error)),
                 this, SLOT(sourceErrorReceived(QGeoPositionInfoSource::Error)));
+        connect(m_positionSource, SIGNAL(updateTimeout()),
+                this, SLOT(updateTimeoutReceived()));
 
         m_positionSource->setUpdateInterval(m_updateInterval);
         m_positionSource->setPreferredPositioningMethods(
@@ -248,8 +263,10 @@ void QDeclarativePositionSource::setNmeaSource(const QUrl &nmeaSource)
                 localFileName.remove(0, 7);
             } else if (localFileName.startsWith("file:///")) {
                 localFileName.remove(0, 7);
+            } else if (localFileName.startsWith("qrc:/")) {
+                localFileName.remove(0, 5);
             }
-            if (!QFile::exists(localFileName) && localFileName.startsWith("/")) {
+            if (!QFile::exists(localFileName) && localFileName.startsWith('/')) {
                 localFileName.remove(0,1);
             }
         }
@@ -280,16 +297,22 @@ void QDeclarativePositionSource::setNmeaSource(const QUrl &nmeaSource)
             qDebug() << "QDeclarativePositionSource NMEA File was found: " << localFileName;
 #endif
             m_positionSource = new QNmeaPositionInfoSource(QNmeaPositionInfoSource::SimulationMode);
+            (qobject_cast<QNmeaPositionInfoSource *>(m_positionSource))->setUserEquivalentRangeError(2.5); // it is internally multiplied by 2 in qlocationutils_readGga
             (qobject_cast<QNmeaPositionInfoSource *>(m_positionSource))->setDevice(m_nmeaFile);
             connect(m_positionSource, SIGNAL(positionUpdated(QGeoPositionInfo)),
                     this, SLOT(positionUpdateReceived(QGeoPositionInfo)));
+            connect(m_positionSource, SIGNAL(error(QGeoPositionInfoSource::Error)),
+                    this, SLOT(sourceErrorReceived(QGeoPositionInfoSource::Error)));
+            connect(m_positionSource, SIGNAL(updateTimeout()),
+                    this, SLOT(updateTimeoutReceived()));
+
             setPosition(m_positionSource->lastKnownPosition());
             if (m_active && !m_singleUpdate) {
                 // Keep on updating even though source changed
                 QTimer::singleShot(0, this, SLOT(start()));
             }
         } else {
-            qmlInfo(this) << QStringLiteral("Nmea file not found") << localFileName;
+            qmlWarning(this) << QStringLiteral("Nmea file not found") << localFileName;
 #ifdef QDECLARATIVE_POSITION_DEBUG
             qDebug() << "QDeclarativePositionSource NMEA File was not found: " << localFileName;
 #endif
@@ -329,6 +352,10 @@ void QDeclarativePositionSource::socketConnected()
 
     connect(m_positionSource, &QNmeaPositionInfoSource::positionUpdated,
             this, &QDeclarativePositionSource::positionUpdateReceived);
+    connect(m_positionSource, SIGNAL(error(QGeoPositionInfoSource::Error)),
+            this, SLOT(sourceErrorReceived(QGeoPositionInfoSource::Error)));
+    connect(m_positionSource, SIGNAL(updateTimeout()),
+            this, SLOT(updateTimeoutReceived()));
 
     setPosition(m_positionSource->lastKnownPosition());
 
@@ -368,6 +395,24 @@ void QDeclarativePositionSource::socketError(QAbstractSocket::SocketError error)
     emit sourceErrorChanged();
 }
 
+
+void QDeclarativePositionSource::updateTimeoutReceived()
+{
+    if (!m_active)
+        return;
+
+    if (m_singleUpdate) {
+        m_singleUpdate = false;
+
+        // only singleUpdate based timeouts change activity
+        // continuous updates may resume again (see QGeoPositionInfoSource::startUpdates())
+        m_active = false;
+        emit activeChanged();
+    }
+
+    emit updateTimeout();
+}
+
 void QDeclarativePositionSource::setPosition(const QGeoPositionInfo &pi)
 {
     m_position.setPosition(pi);
@@ -515,10 +560,9 @@ QDeclarativePositionSource::PositioningMethods QDeclarativePositionSource::prefe
 
 void QDeclarativePositionSource::start()
 {
-    if (!m_positionSource)
-        return;
+    if (m_positionSource)
+        m_positionSource->startUpdates();
 
-    m_positionSource->startUpdates();
     if (!m_active) {
         m_active = true;
         emit activeChanged();
@@ -669,12 +713,20 @@ void QDeclarativePositionSource::componentComplete()
                     this, SLOT(positionUpdateReceived(QGeoPositionInfo)));
             connect(m_positionSource, SIGNAL(error(QGeoPositionInfoSource::Error)),
                     this, SLOT(sourceErrorReceived(QGeoPositionInfoSource::Error)));
+            connect(m_positionSource, SIGNAL(updateTimeout()),
+                    this, SLOT(updateTimeoutReceived()));
 
             m_positionSource->setUpdateInterval(m_updateInterval);
             m_positionSource->setPreferredPositioningMethods(
                 static_cast<QGeoPositionInfoSource::PositioningMethods>(int(m_preferredPositioningMethods)));
 
             setPosition(m_positionSource->lastKnownPosition());
+
+            if (m_active)
+                QTimer::singleShot(0, this, SLOT(start())); // delay ensures all properties have been set
+        } else if (m_active) {
+            m_active = false;
+            emit activeChanged();
         }
 
         if (previousUpdateInterval != updateInterval())
@@ -687,12 +739,6 @@ void QDeclarativePositionSource::componentComplete()
             emit supportedPositioningMethodsChanged();
 
         emit validityChanged();
-
-        if (m_active) {
-            m_active = false;
-            emit activeChanged();
-        }
-
         emit nameChanged();
     }
 }
@@ -714,6 +760,4 @@ void QDeclarativePositionSource::sourceErrorReceived(const QGeoPositionInfoSourc
     emit sourceErrorChanged();
 }
 
-#include "moc_qdeclarativepositionsource_p.cpp"
-
 QT_END_NAMESPACE
