@@ -150,8 +150,9 @@ void QGeoMappingManagerEngine::setSupportedMapTypes(const QList<QGeoMapType> &su
     emit supportedMapTypesChanged();
 }
 
-QGeoCameraCapabilities QGeoMappingManagerEngine::cameraCapabilities() const
+QGeoCameraCapabilities QGeoMappingManagerEngine::cameraCapabilities(int mapId) const
 {
+    Q_UNUSED(mapId)
     Q_D(const QGeoMappingManagerEngine);
     return d->capabilities_;
 }
@@ -203,6 +204,4 @@ QGeoMappingManagerEnginePrivate::QGeoMappingManagerEnginePrivate()
 
 QGeoMappingManagerEnginePrivate::~QGeoMappingManagerEnginePrivate() {}
 
-#include "moc_qgeomappingmanagerengine_p.cpp"
-
 QT_END_NAMESPACE
