@@ -48,7 +48,7 @@ QT_BEGIN_NAMESPACE
     \class QGeoRoute
     \inmodule QtLocation
     \ingroup QtLocation-routing
-    \since 5.5
+    \since 5.6
 
     \brief The QGeoRoute class represents a route between two points.
 
@@ -104,7 +104,7 @@ QGeoRoute &QGeoRoute::operator= (const QGeoRoute & other)
 */
 bool QGeoRoute::operator ==(const QGeoRoute &other) const
 {
-    return (d_ptr.constData() == other.d_ptr.constData());
+    return (*d_ptr.constData() == *other.d_ptr.constData());
 }
 
 /*!
@@ -112,7 +112,7 @@ bool QGeoRoute::operator ==(const QGeoRoute &other) const
 */
 bool QGeoRoute::operator !=(const QGeoRoute &other) const
 {
-    return (d_ptr.constData() != other.d_ptr.constData());
+    return !(*d_ptr.constData() == *other.d_ptr.constData());
 }
 
 /*!
