@@ -95,6 +95,7 @@ static QString cardinalDirection4(QLocationUtils::CardinalDirection direction)
 {
     switch (direction) {
         case QLocationUtils::CardinalN:
+            //: Translations exist at https://github.com/Project-OSRM/osrm-text-instructions.
             //: Always used in "Head %1 [onto <street name>]"
             return QGeoRouteParserOsrmV5::tr("North");
         case QLocationUtils::CardinalE:
@@ -428,9 +429,9 @@ static QString instructionNewName(const QString &wayName, QGeoManeuver::Instruct
         case QGeoManeuver::DirectionLightRight:
         case QGeoManeuver::DirectionBearRight:
             if (wayName.isEmpty())
-                return QGeoRouteParserOsrmV5::tr("Contine slightly right");
+                return QGeoRouteParserOsrmV5::tr("Continue slightly right");
             else
-                return QGeoRouteParserOsrmV5::tr("Contine slightly right onto %1").arg(wayName);
+                return QGeoRouteParserOsrmV5::tr("Continue slightly right onto %1").arg(wayName);
         case QGeoManeuver::DirectionUTurnLeft:
         case QGeoManeuver::DirectionUTurnRight:
             if (wayName.isEmpty())
@@ -468,9 +469,9 @@ static QString instructionNotification(const QString &wayName, QGeoManeuver::Ins
         case QGeoManeuver::DirectionLightRight:
         case QGeoManeuver::DirectionBearRight:
             if (wayName.isEmpty())
-                return QGeoRouteParserOsrmV5::tr("Contine on the right");
+                return QGeoRouteParserOsrmV5::tr("Continue on the right");
             else
-                return QGeoRouteParserOsrmV5::tr("Contine on the right on %1").arg(wayName);
+                return QGeoRouteParserOsrmV5::tr("Continue on the right on %1").arg(wayName);
         case QGeoManeuver::DirectionForward:
         default:
             if (wayName.isEmpty())
@@ -590,11 +591,9 @@ static QString instructionRoundaboutTurn(const QString &wayName, QGeoManeuver::I
 
 static QString instructionTrain(const QString &wayName)
 {
-    QString instruction = QGeoRouteParserOsrmV5::tr("Take the train");
-    if (!wayName.isEmpty())
-        instruction += QLatin1String(" [") + wayName + QLatin1Char(']');
-
-    return instruction;
+    return wayName.isEmpty()
+        ? QGeoRouteParserOsrmV5::tr("Take the train")
+        : QGeoRouteParserOsrmV5::tr("Take the train [%1]").arg(wayName);
 }
 
 static QString instructionTurn(const QString &wayName, QGeoManeuver::InstructionDirection direction)
@@ -954,7 +953,7 @@ QUrl QGeoRouteParserOsrmV5Private::requestUrl(const QGeoRouteRequest &request, c
     foreach (const QGeoCoordinate &c, request.waypoints()) {
         if (notFirst)
             routingUrl.append(QLatin1Char(';'));
-        routingUrl.append(QString::number(c.longitude())).append(QLatin1Char(',')).append(QString::number(c.latitude()));
+        routingUrl.append(QString::number(c.longitude(), 'f', 7)).append(QLatin1Char(',')).append(QString::number(c.latitude(), 'f', 7));
         ++notFirst;
     }
 
