@@ -47,17 +47,19 @@
 QT_BEGIN_NAMESPACE
 
 class QNetworkAccessManager;
+class QGeoTileFetcherOsmPrivate;
 
 class QGeoTileFetcherOsm : public QGeoTileFetcher
 {
     Q_OBJECT
+    Q_DECLARE_PRIVATE(QGeoTileFetcherOsm)
 
     friend class QGeoMapReplyOsm;
     friend class QGeoTiledMappingManagerEngineOsm;
 public:
     QGeoTileFetcherOsm(const QVector<QGeoTileProviderOsm *> &providers,
                        QNetworkAccessManager *nm,
-                       QObject *parent = 0);
+                       QGeoMappingManagerEngine *parent);
 
     void setUserAgent(const QByteArray &userAgent);
 
@@ -69,10 +71,11 @@ protected:
 
 protected Q_SLOTS:
     void onProviderResolutionFinished(const QGeoTileProviderOsm *provider);
-    void onProviderResolutionError(const QGeoTileProviderOsm *provider, QNetworkReply::NetworkError error);
+    void onProviderResolutionError(const QGeoTileProviderOsm *provider);
+    void restartTimer();
 
 private:
-    QGeoTiledMapReply *getTileImage(const QGeoTileSpec &spec);
+    QGeoTiledMapReply *getTileImage(const QGeoTileSpec &spec) Q_DECL_OVERRIDE;
     void readyUpdated();
 
     QByteArray m_userAgent;
