@@ -41,6 +41,7 @@
 #include <QDebug>
 #include <QMap>
 #include <QtGlobal>
+#include <QtCore/private/qjnihelpers_p.h>
 #include <android/log.h>
 #include <jni.h>
 #include <QGeoPositionInfo>
@@ -76,7 +77,7 @@ namespace AndroidPositioning {
         AttachedJNIEnv()
         {
             attached = false;
-            if (javaVM->GetEnv((void**)&jniEnv, JNI_VERSION_1_6) < 0) {
+            if (javaVM && javaVM->GetEnv((void**)&jniEnv, JNI_VERSION_1_6) < 0) {
                 if (javaVM->AttachCurrentThread(&jniEnv, NULL) < 0) {
                     __android_log_print(ANDROID_LOG_ERROR, logTag, "AttachCurrentThread failed");
                     jniEnv = 0;
@@ -99,7 +100,7 @@ namespace AndroidPositioning {
     {
         static bool firstInit = true;
         if (firstInit) {
-            qsrand( QDateTime::currentDateTime().toTime_t() );
+            qsrand( QDateTime::currentMSecsSinceEpoch() / 1000 );
             firstInit = false;
         }
 
@@ -232,7 +233,7 @@ namespace AndroidPositioning {
         //time stamp
         mid = getCachedMethodID(jniEnv, thisClass, "getTime", "()J");
         jlong timestamp = jniEnv->CallLongMethod(location, mid);
-        info.setTimestamp(QDateTime::fromMSecsSinceEpoch(timestamp));
+        info.setTimestamp(QDateTime::fromMSecsSinceEpoch(timestamp, Qt::UTC));
 
         //accuracy
         mid = getCachedMethodID(jniEnv, thisClass, "hasAccuracy", "()Z");
@@ -367,6 +368,20 @@ namespace AndroidPositioning {
         QGeoPositionInfoSourceAndroid *source = AndroidPositioning::idToPosSource()->value(androidClassKey);
 
         if (source) {
+            // Android v23+ requires runtime permission check and requests
+            QString permission(QLatin1String("android.permission.ACCESS_FINE_LOCATION"));
+
+            if (QtAndroidPrivate::checkPermission(permission) == QtAndroidPrivate::PermissionsResult::Denied) {
+                const QHash<QString, QtAndroidPrivate::PermissionsResult> results =
+                        QtAndroidPrivate::requestPermissionsSync(env.jniEnv, QStringList() << permission);
+                if (!results.contains(permission)
+                    || results[permission] == QtAndroidPrivate::PermissionsResult::Denied)
+                {
+                    qWarning() << "Position retrieval not possible due to missing permission (ACCESS_FINE_LOCATION)";
+                    return QGeoPositionInfoSource::AccessError;
+                }
+            }
+
             int errorCode = env.jniEnv->CallStaticIntMethod(positioningClass, startUpdatesMethodId,
                                              androidClassKey,
                                              positioningMethodToInt(source->preferredPositioningMethods()),
@@ -404,6 +419,20 @@ namespace AndroidPositioning {
         QGeoPositionInfoSourceAndroid *source = AndroidPositioning::idToPosSource()->value(androidClassKey);
 
         if (source) {
+            // Android v23+ requires runtime permission check and requests
+            QString permission(QLatin1String("android.permission.ACCESS_FINE_LOCATION"));
+
+            if (QtAndroidPrivate::checkPermission(permission) == QtAndroidPrivate::PermissionsResult::Denied) {
+                const QHash<QString, QtAndroidPrivate::PermissionsResult> results =
+                        QtAndroidPrivate::requestPermissionsSync(env.jniEnv, QStringList() << permission);
+                if (!results.contains(permission)
+                    || results[permission] == QtAndroidPrivate::PermissionsResult::Denied)
+                {
+                    qWarning() << "Position update not possible due to missing permission (ACCESS_FINE_LOCATION)";
+                    return QGeoPositionInfoSource::AccessError;
+                }
+            }
+
             int errorCode = env.jniEnv->CallStaticIntMethod(positioningClass, requestUpdateMethodId,
                                              androidClassKey,
                                              positioningMethodToInt(source->preferredPositioningMethods()));
