@@ -1,31 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -206,6 +201,7 @@ public:
         m_locales << QLocale();
         if (parameters.value(QStringLiteral("initializePlaceData"), false).toBool()) {
             QFile placeData(QFINDTESTDATA("place_data.json"));
+            QVERIFY(placeData.exists());
             if (placeData.open(QIODevice::ReadOnly)) {
                 QJsonDocument document = QJsonDocument::fromJson(placeData.readAll());
 
@@ -339,7 +335,7 @@ public:
         }
     }
 
-    QPlaceDetailsReply *getPlaceDetails(const QString &placeId)
+    QPlaceDetailsReply *getPlaceDetails(const QString &placeId) Q_DECL_OVERRIDE
     {
         DetailsReply *reply = new DetailsReply(this);
 
@@ -423,7 +419,7 @@ public:
         return reply;
     }
 
-    QPlaceSearchReply *search(const QPlaceSearchRequest &query)
+    QPlaceSearchReply *search(const QPlaceSearchRequest &query) Q_DECL_OVERRIDE
     {
         QList<QPlaceSearchResult> results;
 
@@ -468,7 +464,7 @@ public:
         return reply;
     }
 
-    QPlaceSearchSuggestionReply *searchSuggestions(const QPlaceSearchRequest &query)
+    QPlaceSearchSuggestionReply *searchSuggestions(const QPlaceSearchRequest &query) Q_DECL_OVERRIDE
     {
         QStringList suggestions;
         if (query.searchTerm() == QLatin1String("test")) {
@@ -484,7 +480,7 @@ public:
         return reply;
     }
 
-    QPlaceIdReply *savePlace(const QPlace &place)
+    QPlaceIdReply *savePlace(const QPlace &place) Q_DECL_OVERRIDE
     {
         IdReply *reply = new IdReply(QPlaceIdReply::SavePlace, this);
 
@@ -507,7 +503,7 @@ public:
         return reply;
     }
 
-    QPlaceIdReply *removePlace(const QString &placeId)
+    QPlaceIdReply *removePlace(const QString &placeId) Q_DECL_OVERRIDE
     {
         IdReply *reply = new IdReply(QPlaceIdReply::RemovePlace, this);
         reply->setId(placeId);
@@ -524,7 +520,7 @@ public:
         return reply;
     }
 
-    QPlaceIdReply *saveCategory(const QPlaceCategory &category, const QString &parentId)
+    QPlaceIdReply *saveCategory(const QPlaceCategory &category, const QString &parentId) Q_DECL_OVERRIDE
     {
         IdReply *reply = new IdReply(QPlaceIdReply::SaveCategory, this);
 
@@ -565,7 +561,7 @@ public:
         return reply;
     }
 
-    QPlaceIdReply *removeCategory(const QString &categoryId)
+    QPlaceIdReply *removeCategory(const QString &categoryId) Q_DECL_OVERRIDE
     {
         IdReply *reply = new IdReply(QPlaceIdReply::RemoveCategory, this);
         reply->setId(categoryId);
@@ -588,7 +584,7 @@ public:
         return reply;
     }
 
-    QPlaceReply *initializeCategories()
+    QPlaceReply *initializeCategories() Q_DECL_OVERRIDE
     {
         QPlaceReply *reply = new PlaceReply(this);
 
@@ -597,7 +593,7 @@ public:
         return reply;
     }
 
-    QString parentCategoryId(const QString &categoryId) const
+    QString parentCategoryId(const QString &categoryId) const Q_DECL_OVERRIDE
     {
         QHashIterator<QString, QStringList> i(m_childCategories);
         while (i.hasNext()) {
@@ -609,17 +605,17 @@ public:
         return QString();
     }
 
-    virtual QStringList childCategoryIds(const QString &categoryId) const
+    virtual QStringList childCategoryIds(const QString &categoryId) const Q_DECL_OVERRIDE
     {
         return m_childCategories.value(categoryId);
     }
 
-    virtual QPlaceCategory category(const QString &categoryId) const
+    virtual QPlaceCategory category(const QString &categoryId) const Q_DECL_OVERRIDE
     {
         return m_categories.value(categoryId);
     }
 
-    QList<QPlaceCategory> childCategories(const QString &parentId) const
+    QList<QPlaceCategory> childCategories(const QString &parentId) const Q_DECL_OVERRIDE
     {
         QList<QPlaceCategory> categories;
 
@@ -629,17 +625,18 @@ public:
         return categories;
     }
 
-    QList<QLocale> locales() const
+    QList<QLocale> locales() const Q_DECL_OVERRIDE
     {
         return m_locales;
     }
 
-    void setLocales(const QList<QLocale> &locales)
+    void setLocales(const QList<QLocale> &locales) Q_DECL_OVERRIDE
     {
         m_locales = locales;
     }
 
-    QUrl constructIconUrl(const QPlaceIcon &icon, const QSize &size) const {
+    QUrl constructIconUrl(const QPlaceIcon &icon, const QSize &size) const Q_DECL_OVERRIDE
+    {
         QList<QPair<int, QUrl> > candidates;
 
         QMap<QString, int> sizeDictionary;
@@ -675,7 +672,8 @@ public:
         }
     }
 
-    QPlace compatiblePlace(const QPlace &original) const {
+    QPlace compatiblePlace(const QPlace &original) const Q_DECL_OVERRIDE
+    {
         QPlace place;
         place.setName(original.name());
         return place;
