@@ -1,39 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -331,16 +318,16 @@ void tst_nokia_routing::routingError(QGeoRouteReply* reply, QGeoRouteReply::Erro
 void tst_nokia_routing::initTestCase()
 {
     QStringList providers = QGeoServiceProvider::availableServiceProviders();
-    QVERIFY(providers.contains(QStringLiteral("nokia")));
+    QVERIFY(providers.contains(QStringLiteral("here")));
 
     m_networkManager = new MockGeoNetworkAccessManager();
 
-    QMap<QString, QVariant> parameters;
+    QVariantMap parameters;
     parameters.insert(QStringLiteral("nam"), QVariant::fromValue<void*>(m_networkManager));
-    parameters.insert(QStringLiteral("app_id"), "stub");
-    parameters.insert(QStringLiteral("token"), "stub");
+    parameters.insert(QStringLiteral("here.app_id"), "stub");
+    parameters.insert(QStringLiteral("here.token"), "stub");
 
-    m_geoServiceProvider = new QGeoServiceProvider(QStringLiteral("nokia"), parameters);
+    m_geoServiceProvider = new QGeoServiceProvider(QStringLiteral("here"), parameters);
     QVERIFY(m_geoServiceProvider);
 
     m_routingManager = m_geoServiceProvider->routingManager();
@@ -410,11 +397,10 @@ void tst_nokia_routing::can_compute_route_for_all_supported_travel_modes_data()
 void tst_nokia_routing::can_compute_route_for_all_supported_optimizations()
 {
     QFETCH(int, optimization);
-    Q_UNUSED(optimization);
     QFETCH(QString, file);
     QFETCH(qreal, distance);
     QFETCH(int, duration);
-
+    m_dummyRequest.setRouteOptimization((QGeoRouteRequest::RouteOptimization)optimization);
     loadReply(file);
     calculateRoute();
     QList<QGeoRoute> routes = m_reply->routes();
@@ -434,11 +420,8 @@ void tst_nokia_routing::can_compute_route_for_all_supported_optimizations_data()
 
     QTest::newRow("Shortest") << (int)QGeoRouteRequest::ShortestRoute << QString("optim-shortest.xml") << qreal(1177.0) << 309;
     QTest::newRow("Fastest") << (int)QGeoRouteRequest::FastestRoute << QString("optim-fastest.xml") << qreal(1271.0) << 243;
-    QTest::newRow("Most Econonic") << (int)QGeoRouteRequest::MostEconomicRoute << QString("optim-economic.xml") << qreal(1177.0) << 309;
-    QTest::newRow("Most Scenic") << (int)QGeoRouteRequest::MostScenicRoute << QString("optim-scenic.xml") << qreal(1177.0) << 309;
 }
 
-
 void tst_nokia_routing::can_handle_multiple_routes_in_response()
 {
     loadReply(QStringLiteral("multiple-routes-in-response.xml"));
@@ -525,11 +508,10 @@ void tst_nokia_routing::foobar_data()
     QTest::newRow("QNetworkReply::UnknownNetworkError") << int(QNetworkReply::UnknownNetworkError);
     QTest::newRow("QNetworkReply::UnknownProxyError") << int(QNetworkReply::UnknownProxyError);
     QTest::newRow("QNetworkReply::ProxyAuthenticationRequiredError") << int(QNetworkReply::ProxyAuthenticationRequiredError);
-    QTest::newRow("QNetworkReply::UnknownContentError") << int(QNetworkReply::UnknownContentError);
     QTest::newRow("QNetworkReply::ProtocolFailure") << int(QNetworkReply::ProtocolFailure);
 }
 
 
-QTEST_APPLESS_MAIN(tst_nokia_routing)
+QTEST_MAIN(tst_nokia_routing)
 
 #include "tst_routing.moc"
